/*
    CaPriCe Forever - Amstrad CPC 464/664/6128 emulator
    Copyright (C) 2015-2019 by Frdric Coste

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "..\Native_CPC.h"


/*
** Make sure we can call this stuff from C++.
*/
#ifdef __cplusplus
extern "C" {
#endif


//===================
// PATCH begin
#ifdef _PATCH_ENABLE

#endif /* _PATCH_ENABLE */
// PATCH end
//===================


#define FILE_ENTRY_SIZE                32


tULong AMSDOS_GetFirstSectorOfBlock(tDiskOperation* DiskOperation, tULong Block);
tULong AMSDOS_GetNextSector(tDiskOperation* DiskOperation, tULong CHRN);


tULong AMSDOS_DiskGetFirstSector(tDiskOperation* DiskOperation)
/***********************************************************************
 *
 *  AMSDOS_DiskGetFirstSector
 *
 ***********************************************************************/
{
tTrack* actualTrackP;
tSector* sectorP;
tULong indexSector;
tULong loopSector;
tULong loopEntry;
tULong loopChar;
tULong CHRN;

  actualTrackP = &DiskOperation->Drive->track[DiskOperation->CatalogueTrack][0]; // Side = 0

  // Only find file entries into four first sectors.
  indexSector = 1; // Begin to find the first sector
  CHRN = 0L;
  while ( (indexSector <= 4) && (CHRN == 0L) )
  {
    tULong nbofentries;
    tUChar* fileEntryP;

    //
    // Find sector
    //
    sectorP = 0;
    for (loopSector=0; loopSector < actualTrackP->sectors; loopSector++)
    {
      if ( ((actualTrackP->sector[loopSector].CHRN[SECTOR_CHRN_R] & 0x0FU) == indexSector)
           && (actualTrackP->sector[loopSector].CHRN[SECTOR_CHRN_N] == 2) )
      {
        sectorP = &actualTrackP->sector[loopSector];
        break;
      }
    }

  	// Do not take into account empty or missing sector
  	if (!sectorP || (!sectorP->size) )
  	{
  		// Next sector
  		indexSector++;
  	 	continue;
  	}

    // Prepare sector read
    nbofentries = sectorP->declared_size / FILE_ENTRY_SIZE; // 32 bytes per file entry
  	fileEntryP = sectorP->data; // Point at beginning of sector.

    for (loopEntry=0; loopEntry < nbofentries; loopEntry++, fileEntryP += FILE_ENTRY_SIZE)
    {
      tUChar* bytePtr;
      tUChar* fileStoredEntryP;
      
      //
      // Parse File Entry
      //
      // Entry documentation
      // ===================
      // Byte 0x0 = USER; 229(0xe5) for deleted files, 0 by default.
      // Byte 0x1 to 0x8 = filename
      // Byte 0x9 to 0xb = extension. Byte 0x9 bit7 = 1 = Read Only. Byte 0xa bit7 = 1 = Hidden.
      // Byte 0xc = entry index (First entry = 0)
      // Byte 0xd to 0xe = Not used ??
      // Byte 0xf = size in records (1 record=128 bytes). 0x80 for full entry => check another entry
      // Byte 0x10 to 0x1f = list of numbers of 1k blocks (2 sectors length).

      // How to locate a block
      // =====================
      // Cylinder (C) = INT(block * 2 / 9); 9 sectors per track !! For EXTENDED, check number of sectors per Tracks
      // Head     (H) = Always 0
      // Sector   (R) = (block * 2) MODULO 9 + 1 + TYPE (0xC0 for DATA, 0x40 for VENDOR)
      // Size     (N) = Size (*256 bytes)

      // Entry size is non null
      if (!fileEntryP[0xf])
        continue;

      // First entry ?
      if (fileEntryP[0xc])
        continue;

    	// Compare filename and extension
  	  fileStoredEntryP = DiskOperation->CatalogueEntries[DiskOperation->CatalogueEntryIndex];
    	bytePtr = &fileEntryP[0x1]; // Point at the beginning of the filename.
    	for (loopChar=0; loopChar < 11; loopChar++)
    	{
    		if ((*bytePtr++ & 0x7f) != *fileStoredEntryP++)
    		  break;
      }
      // Matching filename ?
      // Matching USER
      if ( (loopChar == 11)
           && (fileEntryP[0x0] == DiskOperation->CatalogueEntryUser[DiskOperation->CatalogueEntryIndex]) )
      {
        CHRN = AMSDOS_GetFirstSectorOfBlock(DiskOperation, fileEntryP[0x10]);
        break;
      }
    }

    indexSector++; // Next sector
  }

  DiskOperation->CatalogueLastCHRN = CHRN;
  
  return CHRN;
}
/*----------------------------------------------------------------------------*/


tULong AMSDOS_GetFirstSectorOfBlock(tDiskOperation* DiskOperation, tULong Block)
/***********************************************************************
 *
 *  AMSDOS_GetFirstSectorOfBlock
 *
 ***********************************************************************/
{
  tULong CHRN;
  tUChar Cylinder;
  tUChar Head;
  tUChar Sector;
  tUChar Size;

  Head = 0;
  Size = 2; // 512 bytes

  // AMSDOS SYSTEM
  if (DiskOperation->FormatType == DISK_FORMAT_AMSDOS_SYSTEM)
  {
    Cylinder = (Block*2) / 9;
    Sector = ((Block*2) - (Cylinder * 9) + 1) | 0x40;
  }
  // AMSDOS IBM
  else if (DiskOperation->FormatType == DISK_FORMAT_AMSDOS_IBM)
  {
    Cylinder = (Block*2) / 8;
    Sector = ((Block*2) - (Cylinder * 9) + 1) | 0x00;
  }
  // AMSDOS DATA
  else
  {
    Cylinder = (Block*2) / 9;
    Sector = ((Block*2) - (Cylinder * 9) + 1) | 0xC0;
  }

  CHRN  = (Cylinder + DiskOperation->CatalogueTrack) << 24;
  CHRN |= Head << 16;
  CHRN |= Sector << 8;
  CHRN |= Size << 0;

  return CHRN; 
}
/*----------------------------------------------------------------------------*/


tULong AMSDOS_GetNextSector(tDiskOperation* DiskOperation, tULong CHRN)
/***********************************************************************
 *
 *  AMSDOS_GetNextSector
 *
 ***********************************************************************/
{
  tUChar NbSectors;
  tUChar Cylinder;
  tUChar Head;
  tUChar Sector;
  tUChar Size;

  Cylinder = (CHRN >> 24) & 0xff;
  Head     = (CHRN >> 16) & 0xff;
  Sector   = (CHRN >> 8) & 0xff;
  Size     = (CHRN >> 0) & 0xff;

  // AMSDOS SYSTEM
  if (DiskOperation->FormatType == DISK_FORMAT_AMSDOS_SYSTEM)
  {
    NbSectors = 9;
  }
  // AMSDOS IBM
  else if (DiskOperation->FormatType == DISK_FORMAT_AMSDOS_IBM)
  {
    NbSectors = 8;
  }
  // AMSDOS DATA
  else
  {
    NbSectors = 9;
  }

  // Increment Sector
  Sector++;
  if ( (Sector & 0x0f) > NbSectors)
  {
    // First sector of next track
    Sector = (Sector & 0xf0) + 1;
    Cylinder++;
  }
  
  CHRN  = Cylinder << 24;
  CHRN |= Head << 16;
  CHRN |= Sector << 8;
  CHRN |= Size << 0;

  return CHRN; 
}
/*----------------------------------------------------------------------------*/


tSector* AMSDOS_FindSector(tDiskOperation* DiskOperation, tULong CHRN)
/***********************************************************************
 *
 *  AMSDOS_FindSector
 *
 ***********************************************************************/
{
  tSector* FoundSector;
  tTrack* TrackP;
  tUChar loopSector;
  tUChar Cylinder;
  tUChar Head;
  tUChar Sector;
  tUChar Size;

  Cylinder = (CHRN >> 24) & 0xff;
  Head     = (CHRN >> 16) & 0xff;
  Sector   = (CHRN >> 8) & 0xff;
  Size     = (CHRN >> 0) & 0xff;

  // Find sector
  FoundSector = NULL;
  TrackP = &DiskOperation->Drive->track[Cylinder][Head];
  for (loopSector=0; loopSector < (tUChar)TrackP->sectors; loopSector++)
  {
    tSector* SectorP = &TrackP->sector[loopSector];
    if ( (SectorP->CHRN[SECTOR_CHRN_R] == Sector)
         && (SectorP->CHRN[SECTOR_CHRN_N] == Size) )
    {
      FoundSector = SectorP;
      break;
    }
  }

  return FoundSector;
}
/*----------------------------------------------------------------------------*/


#ifdef __cplusplus
}  /* End of the 'extern "C"' block */
#endif
