/*
    CaPriCe Forever - Amstrad CPC 464/664/6128 emulator
    Copyright (C) 2015-2019 by Frdric Coste

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "..\Native_CPC.h"


/*
** Make sure we can call this stuff from C++.
*/
#ifdef __cplusplus
extern "C" {
#endif


//===================
// PATCH begin
#ifdef _PATCH_ENABLE

#endif /* _PATCH_ENABLE */
// PATCH end
//===================



#define FILE_ENTRY_SIZE                32


tULong AMSDOS_DiskGetFreeCatEntries(tDiskOperation* DiskOperation)
/***********************************************************************
 *
 *  AMSDOS_DiskGetFreeCatEntries
 *
 ***********************************************************************/
{
tDrive* driveP;
tTrack* actualTrackP;
tSector* sectorP;
tULong indexSector;
tULong loopSector;
tULong numberEntries;

  numberEntries = 0;

  driveP = DiskOperation->Drive;
  if (DiskOperation->FormatType == DISK_FORMAT_AMSDOS_SYSTEM)
  {
    // Catalogue in track 2
    actualTrackP = &driveP->track[2][0];
  }
  else
  {
    // Catalogue in track 0
    actualTrackP = &driveP->track[0][0];
  }

  do
  {
    // Do not take into account empty or missing tracks
    if (!actualTrackP->size)
      continue;

    // Only find file entries into four first sectors.
    indexSector = 1; // Begin to find the first sector
    while (indexSector <= 4)
    {
      tULong loopEntry;
      tULong nbofentries;
  	  tUChar* fileEntryP;

      //
      // Find sector
      //
      sectorP = 0;
      for (loopSector=0; loopSector < actualTrackP->sectors; loopSector++)
      {
    	  if ( ((actualTrackP->sector[loopSector].CHRN[SECTOR_CHRN_R] & 0x0FU) == indexSector)
             && (actualTrackP->sector[loopSector].CHRN[SECTOR_CHRN_N] == 2) )
    	  {
          sectorP = &actualTrackP->sector[loopSector];
          break;
    	  }
      }

  	  // Do not take into account empty or missing sector
  	  if (!sectorP || (!sectorP->size) )
  	  {
  	  	// Next sector
  	  	indexSector++;
  	  	continue;
  	  }

      // Prepare sector read
      nbofentries = sectorP->declared_size / FILE_ENTRY_SIZE; // 32 bytes per file entry
  	  fileEntryP = sectorP->data; // Point at beginning of sector.

      for (loopEntry=0; loopEntry < nbofentries; loopEntry++, fileEntryP += FILE_ENTRY_SIZE)
      {
        tUChar* bytePtr;
        tUChar loopBlocks;

        //
        // Parse File Entry
        //
        // Entry documentation
        // ===================
        // Byte 0x0 = USER; 0-15, 229(0xe5) for deleted files, 0 by default.
        // Byte 0x1 to 0x8 = filename
        // Byte 0x9 to 0xb = extension. Byte 0x9 bit7 = 1 = Read Only. Byte 0xa bit7 = 1 = Hidden.
        // Byte 0xc = entry index (First entry = 0)
        // Byte 0xd to 0xe = Not used ??
        // Byte 0xf = size in records (1 record=128 bytes). 0x80 for full entry => check another entry
        // Byte 0x10 to 0x1f = list of numbers of 1k blocks (2 sectors length).

        // How to locate a block
        // =====================
        // Cylinder (C) = INT(block * 2 / 9); 9 sectors per track !! For EXTENDED, check number of sectors per Tracks
        // Head     (H) = Always 0
        // Sector   (R) = (block * 2) MODULO 9 + 1 + TYPE (0xC0 for DATA, 0x40 for VENDOR)
        // Size     (N) = Size (*256 bytes)

        // Valid USER ?
        if (fileEntryP[0x0] != 0xe5)
          continue;

        // Entry is free !!
        numberEntries++;
      }

      indexSector++; // Next sector
    }
  }
  while (0);

  return (numberEntries);
}
/*----------------------------------------------------------------------------*/

#ifdef __cplusplus
}  /* End of the 'extern "C"' block */
#endif
