/*
    CaPriCe Forever - Amstrad CPC 464/664/6128 emulator
    Copyright (C) 2015-2019 by Frdric Coste

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "..\Native_CPC.h"


/*
** Make sure we can call this stuff from C++.
*/
#ifdef __cplusplus
extern "C" {
#endif


//===================
// PATCH begin
#ifdef _PATCH_ENABLE

#endif /* _PATCH_ENABLE */
// PATCH end
//===================



#define FILE_ENTRY_SIZE                32


tBool ROMDOS_DiskGetNextFreeCatEntry(tDiskOperation* DiskOperation)
/***********************************************************************
 *
 *  ROMDOS_DiskGetNextFreeCatEntry
 *
 ***********************************************************************/
{
tDrive* driveP;
tTrack* actualTrackP;
tUChar* foundEntryP;
tSector* sectorP;
tULong indexSector;
tULong loopSector;

  foundEntryP = cNull;

  driveP = DiskOperation->Drive;
  // Catalogue in track 0
  actualTrackP = &driveP->track[0][0];

  do
  {
    // Do not take into account empty or missing tracks
    if (!actualTrackP->size)
      continue;

    // Only find file entries into four first sectors.
    indexSector = 1; // Begin to find the first sector
    while ( (indexSector <= 8) && (foundEntryP == cNull) )
    {
  	  tUChar* fileEntryP;
      tULong loopEntry;
      tULong nbofentries;

      //
      // Find sector
      //
      sectorP = 0;
      for (loopSector=0; loopSector < actualTrackP->sectors; loopSector++)
      {
    	  if ( ((actualTrackP->sector[loopSector].CHRN[SECTOR_CHRN_R] & 0x0FU) == indexSector)
             && (actualTrackP->sector[loopSector].CHRN[SECTOR_CHRN_N] == 2) )
    	  {
          sectorP = &actualTrackP->sector[loopSector];
          break;
    	  }
      }

  	  // Do not take into account empty or missing sector
  	  if (!sectorP || (!sectorP->size) )
  	  {
  	  	// Next sector
  	  	indexSector++;
  	  	continue;
  	  }

      // Prepare sector read
      nbofentries = sectorP->declared_size / FILE_ENTRY_SIZE; // 32 bytes per file entry
  	  fileEntryP = sectorP->data; // Point at beginning of sector.

      for (loopEntry=0; loopEntry < nbofentries; loopEntry++, fileEntryP += FILE_ENTRY_SIZE)
      {
        tUChar* bytePtr;
        tUChar loopBlocks;

        //
        // Parse File Entry
        //
        // Entry documentation
        // ===================
        // Byte 0x0 = USER; 0-15, 229(0xe5) for deleted files, 0 by default.
        // Byte 0x1 to 0x8 = filename
        // Byte 0x9 to 0xb = extension. Byte 0x9 bit7 = 1 = Read Only. Byte 0xa bit7 = 1 = Hidden.
        // Byte 0xc = entry index (First entry = 0)
        // Byte 0xd to 0xe = Not used ??
        // Byte 0xf = size in records (1 record=128 bytes). 0x80 for full entry => check another entry
        // Byte 0x10 to 0x1f = list of numbers of 1k blocks (2 sectors length).

        // How to locate a block
        // =====================
        // Cylinder (C) = INT(block * 2 / 9); 9 sectors per track !! For EXTENDED, check number of sectors per Tracks
        // Head     (H) = Always 0
        // Sector   (R) = (block * 2) MODULO 9 + 1 + TYPE (0xC0 for DATA, 0x40 for VENDOR)
        // Size     (N) = Size (*256 bytes)

        // Free entry found ?
        if ( (fileEntryP[0x0] == 0xe5) && (fileEntryP[0x10] == 0xe5) )
        {
          foundEntryP = fileEntryP;
          break;
        }
        // Deleted entry found ?
        else if ( (fileEntryP[0x0] == 0xe5) && (foundEntryP == cNull) )
        {
          foundEntryP = fileEntryP;
        }
      }

      // Next sector
  	  indexSector++;
    }

    // Entry found ?
    if (foundEntryP != cNull)
    {
      tULong loopBytes;
      
      DiskOperation->CurrentCatEntryP = foundEntryP;
      // Initialise entry
      for (loopBytes=0; loopBytes < FILE_ENTRY_SIZE; loopBytes++)
      {
        foundEntryP[loopBytes] = 0;
      }
      // Add filename with extension.
      for (loopBytes=0; loopBytes < 11; loopBytes++)
      {
        foundEntryP[1 + loopBytes] = DiskOperation->FileName[loopBytes];
      }
      // Add entry index
      foundEntryP[12] = (tUChar)DiskOperation->CatalogueEntryIndex;
      // Update sector data
      sectorP->empty = 0;
      // Signal drive content change
      DiskOperation->Drive->signal_updated++;
      DiskOperation->Drive->signal_content_change++;
      DiskOperation->Drive->signal_sector_write++;
    }
  }
  while (0);

  return (foundEntryP != cNull ? cTrue : cFalse);
}
/*----------------------------------------------------------------------------*/

#ifdef __cplusplus
}  /* End of the 'extern "C"' block */
#endif
